%% Initialize the environment

clc; clear; format compact; format long;

globalNumber = 150;
dataFileNumber = 6;

R = 0.04;
epsilon = 0.05;
scenarioNumberToAssetNumberRatio = 1.5;

averageReturnMax = 0.10;

covarianceMatrixMax = 0.25;
varianceSparsity = 0.6;
covarianceGenerationTempRange = 10;


% Select a seed for random numbers
mySeed = globalNumber;
% RandStream.setDefaultStream( RandStream( 'mt19937ar', 'seed', mySeed ) ); 
RandStream.setGlobalStream( RandStream( 'mt19937ar', 'seed', mySeed ) );

%%

assetNumber = globalNumber;
scenarioNumber = floor( scenarioNumberToAssetNumberRatio * assetNumber );
K = floor( scenarioNumber * epsilon );

for iIndex = 1 : dataFileNumber

    % Randomly generate a return vector
    
    % returnVector = zeros( assetNumber, 1 );
    returnVector = rand( assetNumber, 1 ) * averageReturnMax;
    
    % Randomly generate a covariance matrix
    
    % covarianceMatrix = zeros( assetNumber, assetNumber );
    covarianceMatrix =  randi( [ - covarianceGenerationTempRange, covarianceGenerationTempRange ], assetNumber, assetNumber );
    covarianceMatrix = covarianceMatrix * diag( 1 - rand( assetNumber, 1 ) * varianceSparsity );
    covarianceMatrix = covarianceMatrix' * covarianceMatrix;
    covarianceMatrix = covarianceMatrixMax * covarianceMatrix / max( diag( covarianceMatrix ) );

    % Randomly generate scenarios
    
    scenarioSelection = zeros( assetNumber, scenarioNumber );
    for jIndex = 1 : scenarioNumber
        scenarioSelection( :, jIndex ) = mvnrnd( returnVector, covarianceMatrix );
    end
    
    % Find alpha and beta ( Subject to e'*x=1 and 0<=x<=0.5e )

    sortedScenarioSelection = sort( scenarioSelection );
    alpha = 0.5 * sortedScenarioSelection( 1, : ) + 0.5 * sortedScenarioSelection( 2, : );
    beta = 0.5 * sortedScenarioSelection( assetNumber, : ) + 0.5 * sortedScenarioSelection( assetNumber - 1, : );
    
    % Write data to file
    
    fileID = [ 'set-e' num2str( epsilon ) '-R' num2str( R ) '-n' num2str( globalNumber ) '-' num2str( iIndex ) '.txt' ];

    tempFileId = fopen( fileID, 'w' );
    fprintf( tempFileId, '%d\t%d\t%d\t%f\n', assetNumber, scenarioNumber, K, R );
    fclose( tempFileId );

    dlmwrite( fileID, [ alpha; beta ], '-append', 'delimiter', '\t', 'precision', 8 );
    dlmwrite( fileID, [ ones( 1, assetNumber ) * 0.5; zeros( 1, assetNumber ) ], '-append', 'delimiter', '\t', 'precision', 8 );
    dlmwrite( fileID, covarianceMatrix * 100, '-append', 'delimiter', '\t', 'precision', 8 );
    dlmwrite( fileID, scenarioSelection, '-append', 'delimiter', '\t', 'precision', 8 );

end % End for iIndex



